%****************************************************************************************
% An Integrative Bayesian Modeling Approach to Imaging Genetics
%****************************************************************************************
% (Matlab 2012a required)

% REFERENCE: 
% Francesco C. Stingo, Michele Guindani, Marina Vannucci and Vince D. Calhoun (2013). 
% An Integrative Bayesian Modeling Approach to Imaging Genetics. 
% Journal of the American Statistical Association. 108:503, 876-891.    
% COPYRIGHT: (2013) Francesco Stingo
  
% *****
% USAGE
% *****

% First load the data
load('/home/data.mat');

% in "data.mat" there are the following objects:
% Y - binary disease indicator, training set (1 = healthy control, 0 =
% subject affected by schizophrenia)
% Yf - binary disease indicator, validation set (1 = healthy control, 0 =
% subject affected by schizophrenia)
% X - ROIs, training set (median intensity values)
% Xf - ROIs, validation set (median intensity values)
% Z - SNPs, training set (frequency of minor allele)
% Zf - SNPs, validation set (frequency of minor allele)
% labelsROIs - ROI labels
% labelsSNPs - SNP labels
% Adj_1 - Adjacency matrix for the ROI network


p = size(X,2); % number of ROIs
q = size(Z,2); % number of SNPs

% Set MCMC parameters

gamprec = [];       % Initial set of selected ROIs
deltaprec1 = [];    % Initial set of selected SNPs, for group 1
deltaprec2 = [];    % Initial set of selected SNPs, for group 2
nmetr = 200000;     % Desired number of MCMC iterations
r1 = 2;             % Number of starting variables (ROIs) when 'gamprec' is empty
r2 = 2;             % Nmber of starting variables (SNPs) when 'deltaprec' is empty
Beta01prec = mean(X(Y==0,:),1); % Initial value of $\mu_{0k}$, k = 1
Beta02prec = mean(X(Y==1,:),1); % Initial value of $\mu_{0k}$, k = 2

% Hyperparameters setting
pPar = 0.5;         % probability of the add/delete move (1-pPar = probability of the swap)
% MRF prior on $\gamma$
eta = 0.5;          % Parameter 'e' in the paper
mu = -4.5;          % Parameter 'f' in the paper
% Prior on $\delta$
pd = 0.1;           % Parameter '$w_{rk}$' in the paper
% Prior on $sigma^2_{0j}$
a = 3;              % Parameter '$a_0$' in the paper
b = 0.1;            % Parameter '$b_0$' in the paper
% Prior on $sigma^2_{kj}$
ak = 3;             % Parameter '$a_k$' in the paper
bk = 0.1;           % Parameter '$b_k$' in the paper
% Prior on $\Sigma_k$
c = 0.1;            % Parameter 'c' in the paper
Phi = c*eye(p);     % Parameter 'Q' in the paper
delW = 3;           % Parameter '$d_k$' in the paper
% The prior settings on $\Sigma_k$ imply the following settings for the
% Inv-Gamma Prior on $\sigma_j$'s (see Appendix)
aj = delW/2;        % Parameter $\tilde{a}_j$ in the Appendix
bj = 2/c;           % Parameter $\tilde{b}_j$ in the Appendix
% Prior on $\mu_{0k}$
h = 1;              % Parameter 'h_1^*' in the paper
% Prior on $\beta_{rkj}$
h1 = 1;             % Parameter 'h' in the paper

% Set ROI prior network 
%MRF = 0.*eye(p);   % If no network iformation is used
MRF = Adj_1;        % Matrix S in the paper

% Run the mainprog - Chain #1
[Uni,UniD,GammaD,Uni2_1,Uni2D_1,Uni2_2,Uni2D_2,DeltaD1,DeltaD2,Beta01mat,Beta02mat,logProb,ADgam,Sgam,ADdel1,Sdel1,...
    ADdel2,Sdel2,acceptgam,acceptdelta1,acceptdelta2,acceptBeta1,acceptBeta2,GammaA,DeltaA1,DeltaA2]= ...
mainprogfMRI(X,Y,Z,gamprec,deltaprec1,deltaprec2,nmetr,r1,r2,Beta01prec,Beta02prec,pPar,eta,mu,pd,a,b,ak,bk,Phi,delW,aj,bj,MRF,h,h1);

bi = 1000;
GammaBI1 = GammaA((bi+1):end);
DeltaBI11 = DeltaA1((bi+1):end);
DeltaBI21 = DeltaA2((bi+1):end);
logProb1 = logProb((bi+1):end);
Beta01mat1 = Beta01mat;
Beta02mat1 = Beta02mat;

%Beta01_PostMean_1 = Beta01_PostMean; Beta02_PostMean_1 = Beta02_PostMean;
%MargGam_1 = MargGam; MargDel_11 = MargDel1; MargDel_12 = MargDel2;

clear GammaA DeltaA1 DeltaA2 logProb Beta01mat Beta02mat;
save filename_1.mat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Run the mainprog - Chain #2
[Uni,UniD,GammaD,Uni2_1,Uni2D_1,Uni2_2,Uni2D_2,DeltaD1,DeltaD2,Beta01mat,Beta02mat,logProb,ADgam,Sgam,ADdel1,Sdel1,...
    ADdel2,Sdel2,acceptgam,acceptdelta1,acceptdelta2,acceptBeta1,acceptBeta2,GammaA,DeltaA1,DeltaA2]= ...
mainprogfMRI(X,Y,Z,gamprec,deltaprec1,deltaprec2,nmetr,r1,r2,Beta01prec,Beta02prec,pPar,eta,mu,pd,a,b,ak,bk,Phi,delW,aj,bj,MRF,h,h1);

GammaBI2 = GammaA((bi+1):end);
DeltaBI12 = DeltaA1((bi+1):end);
DeltaBI22 = DeltaA2((bi+1):end);
logProb2 = logProb((bi+1):end);
Beta01mat2 = Beta01mat;
Beta02mat2 = Beta02mat;

clear GammaA DeltaA1 DeltaA2 logProb Beta01mat Beta02mat;
save filename_2.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Run the mainprog - Chain #3
[Uni,UniD,GammaD,Uni2_1,Uni2D_1,Uni2_2,Uni2D_2,DeltaD1,DeltaD2,Beta01mat,Beta02mat,logProb,ADgam,Sgam,ADdel1,Sdel1,...
    ADdel2,Sdel2,acceptgam,acceptdelta1,acceptdelta2,acceptBeta1,acceptBeta2,GammaA,DeltaA1,DeltaA2]= ...
mainprogfMRI(X,Y,Z,gamprec,deltaprec1,deltaprec2,nmetr,r1,r2,Beta01prec,Beta02prec,pPar,eta,mu,pd,a,b,ak,bk,Phi,delW,aj,bj,MRF,h,h1);

GammaBI3 = GammaA((bi+1):end);
DeltaBI13 = DeltaA1((bi+1):end);
DeltaBI23 = DeltaA2((bi+1):end);
logProb3 = logProb((bi+1):end);
Beta01mat3 = Beta01mat;
Beta02mat3 = Beta02mat;

clear GammaA DeltaA1 DeltaA2 logProb Beta01mat Beta02mat;
save filename_3.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Pool together 3 MCMC Chains

GammaD = [GammaBI1 ; GammaBI2; GammaBI3];
DeltaD1 = [DeltaBI11 ; DeltaBI12; DeltaBI13 ];
DeltaD2 = [DeltaBI21 ; DeltaBI22; DeltaBI23];
logProb = [logProb1, logProb2, logProb3];
Beta01mat = [Beta01mat1 Beta01mat2 Beta01mat3];
Beta02mat = [Beta02mat1 Beta02mat2 Beta02mat3];

disp(' ')
disp('------- Calculating marginal probabilities for gamma (ROIs)')
disp(' ')
sss = size(GammaD); GammaBI=GammaD(bi:sss(1)); aa = size(GammaBI);
div = 1; bb = round(aa(1)/div); aaa = 1; freTot = zeros(p,1);

for j=1:div
    bbb = bb*j;
    if j==div 
        bbb=size(GammaBI);
    end
    FreqUni = [];
    for i=aaa:bbb
        FreqUni = [FreqUni str2num(GammaBI{i})];
    end
    fre = tabulate(FreqUni); fsize = size(fre);
    m = p - fsize(1); Z0 = zeros(m,1);
    fre = [fre(:,2);Z0]; freTot = fre+freTot;
    aaa = bbb+1;
end
disp(' ')
disp('------- Selected ROIs ...')
disp(' ')
MargGam = freTot./aa(1); 	
find(MargGam>0.5)

disp(' ')
disp('------- Calculating marginal probabilities for delta1 (SNPs)')
disp(' ')
disp(' ')
disp('------- Freq table ...')
disp(' ')

biD = 20;
sss = size(DeltaD1); DeltaBI1 = DeltaD1(biD:sss(1)); aa = size(DeltaBI1);
div = 1; bb = round(aa(1)/div); aaa = 1; freTotDel1 = zeros(q,1);

for j=1:div
    bbb = bb*j;
    if j==div 
        bbb=size(DeltaBI1);
    end
    FreqUni = [];
    for i=aaa:bbb
        FreqUni = [FreqUni str2num(DeltaBI1{i})];
    end
    fre = tabulate(FreqUni); fsize = size(fre);
    m = q - fsize(1); Z0 = zeros(m,1);
    fre = [fre(:,2);Z0]; freTotDel1 = fre+freTotDel1;
    aaa = bbb+1;
end
disp(' ')
disp('------- Selected SNPs (group 1) ...')
disp(' ')
MargDel1 = freTotDel1./aa(1); find(MargDel1>0.5)'


disp(' ')
disp('------- Calculating marginal probabilities ...')
disp(' ')
MargDel1_p = zeros(1,q); ppp = size(logProb); logProbBI=logProb(biD:sss(1));
postval = logProbBI./sum(logProbBI);

for i=1:length(DeltaBI1)
	numgami=str2num(DeltaBI1{i});
		if ~isempty(numgami)
			MargDel1_p(numgami)=MargDel1_p(numgami)+ ...
            	postval(i)*ones(1,length(numgami));
		else
        end
end
disp(' ')
disp('------- Selected SNPs (group 1) ...')
disp(' ')
find(MargDel1_p>0.5)

disp(' ')
disp('------- Calculating marginal probabilities for delta2 (SNPs)')
disp(' ')
disp(' ')
disp('------- Freq table ...')
disp(' ')

biD = 20;
sss = size(DeltaD2); DeltaBI2 = DeltaD2(biD:sss(1)); aa = size(DeltaBI2);
div = 1; bb = round(aa(1)/div); aaa = 1; freTotDel2 = zeros(q,1);

for j=1:div
    bbb = bb*j;
    if j==div 
        bbb=size(DeltaBI2);
    end
    FreqUni = [];
    for i=aaa:bbb
        FreqUni = [FreqUni str2num(DeltaBI2{i})];
    end
    fre = tabulate(FreqUni); fsize = size(fre);
    m = q - fsize(1); Z0 = zeros(m,1);
    fre = [fre(:,2);Z0]; freTotDel2 = fre+freTotDel2;
    aaa = bbb+1;
end
disp(' ')
disp('------- Selected SNPs (group 2) ...')
disp(' ')
MargDel2 = freTotDel2./aa(1); find(MargDel2>0.5)'


disp(' ')
disp('------- Calculating marginal probabilities ...')
disp(' ')

MargDel2_p = zeros(1,p); ppp = size(logProb); logProbBI=logProb(biD:sss(1));
postval = logProbBI./sum(logProbBI);

for i=1:length(DeltaBI2)
	numgami=str2num(DeltaBI2{i});
		if ~isempty(numgami)
			MargDel2_p(numgami)=MargDel2_p(numgami)+ ...
            	postval(i)*ones(1,length(numgami));
		else
        end
end
disp(' ')
disp('------- Selected SNPs (group 2) ...')
disp(' ')
find(MargDel2_p>0.5)

ROI_sel = find(MargGam>0.5); SNP1_sel = find(MargDel1>0.5); SNP2_sel = find(MargDel2>0.5);
biB = 1000;
Beta01_PostMean = mean(Beta01mat(ROI_sel,biB:end),2);
Beta02_PostMean = mean(Beta02mat(ROI_sel,biB:end),2);

Beta01_gam = []; Beta02_gam = [];
sss = size(GammaD)-1; GammaBI=GammaD(bi:sss(1)); aa = size(GammaBI);
div = 1; bb = round(aa(1)/div); aaa = 1; freTot = zeros(p,1);
bbb=size(GammaBI);
Beta01_gamBI = Beta01mat(ROI_sel,bi:end);
Beta02_gamBI = Beta02mat(ROI_sel,bi:end);
for i=aaa:bbb
    if length(str2num(GammaBI{i}))==length(ROI_sel)
        if str2num(GammaBI{i})==ROI_sel'
            Beta01_gam = [Beta01_gam; Beta01_gamBI(:,i)'];
            Beta02_gam = [Beta02_gam; Beta02_gamBI(:,i)'];
        end
    end    
end
disp(' ')
disp('------- Posterior mean ROIs (group 1) ...')
disp(' ')
mean(Beta01_gam)
disp(' ')
disp('------- Posterior mean ROIs (group 2) ...')
disp(' ')

mean(Beta02_gam)
disp(' ')
disp('------- Correlation among ROIs (group 1) ...')
disp(' ')
corr(Beta01_gam)
disp(' ')
disp('------- Correlation among ROIs (group 2) ...')
disp(' ')
corr(Beta02_gam)
[n p] = size(X); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    Prediction 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ')
disp('------- Prediction  -------')
disp(' ')
soglia =[0.5 ];
soglia2 =[0.5 ];
numErrori = zeros(length(soglia2),length(soglia)); numVar = zeros(length(soglia2),length(soglia));
numSnp1 = zeros(length(soglia2),1); numSnp2 = zeros(length(soglia2),1);
biB = bi;
Y1 = find(Y==0); Y2 = find(Y==1); Y1f = find(Yf==0); Y2f = find(Yf==1);
n1 = sum(Y==0); n2 = sum(Y==1);
G=max(Y)+1;

for jj=1:length(soglia2)
    deltaf1 = MargDel1>soglia2(jj);
    deltaf2 = MargDel2>soglia2(jj);
    nSNPsel1 = sum(deltaf1); nSNPsel2 = sum(deltaf2); ak_p1 = ak + n1/2; ak_p2 = ak + n2/2; akf1 = ak_p1 + 1/2; akf2 = ak_p2 + 1/2;    
    Zd1 = Z(Y1,logical(deltaf1)); Zd2 = Z(Y2,logical(deltaf2)); 
    Zdf1 = Zf(:,logical(deltaf1)); Zdf2 = Zf(:,logical(deltaf2));
    numErroriV = []; numVarV = []; 

    for ii=1:length(soglia)
        gammaf = MargGam>soglia(ii); gammaList = find(gammaf);
        Xgam = X(:,logical(gammaf)); XgamC = X(:,logical(1-gammaf));
        Xfgam = Xf(:,logical(gammaf)); XfgamC = Xf(:,logical(1-gammaf)); Nk =[];
        X1gam = Xgam(Y1,:); X2gam = Xgam(Y2,:); 
        ROI_sel = gammaList ;
        Beta01_PostMean = mean(Beta01mat(ROI_sel,biB:end),2); Beta02_PostMean = mean(Beta02mat(ROI_sel,biB:end),2);
        beta01f = Beta01_PostMean; beta02f = Beta02_PostMean;
        [nf pf] = size(Xf);
        ClassP = []; PpostM = [];

        for i=1:nf        
            AA = 1; BB = 1; Zd1fs = Zdf1(i,:); Zd2fs = Zdf2(i,:);            
            for j=1:sum(gammaf)
                Xj1 = X1gam(:,j); Xj1f = Xfgam(i,j);
                V = inv(eye(nSNPsel1).*1/h+Zd1'*Zd1);
                Vinv = eye(nSNPsel1).*1/h+Zd1'*Zd1;
                M = V*Zd1'*(Xj1-ones(n1,1).*beta01f(j));  
                b1_p = bk + ((Xj1-ones(n1,1).*beta01f(j))'*inv(eye(n1)+Zd1*V*Zd1')*(Xj1-ones(n1,1).*beta01f(j)))/2;                                
                b1f = b1_p+((Xj1f-beta01f(j))*(1/(1+Zd1fs*V*Zd1fs'))*(Xj1f-beta01f(j))+M'*(Vinv-Vinv*inv(Vinv+Zd1fs'*Zd1fs)*Vinv)*M)/2;
                AA = AA*det(V)^(-0.5)*det(Zd1fs'*Zd1fs+inv(V))^(-0.5)*(b1_p^ak_p1)/(b1f^akf1)*gamma(akf1)/gamma(ak_p1);            
                Xj2 = X2gam(:,j); Xj2f = Xfgam(i,j);
                V = inv(eye(nSNPsel2).*1/h+Zd2'*Zd2);
                Vinv = eye(nSNPsel2).*1/h+Zd2'*Zd2;
                M = V*Zd2'*(Xj2-ones(n2,1).*beta02f(j));  
                b2_p = bk + ((Xj2-ones(n2,1).*beta02f(j))'*inv(eye(n2)+Zd2*V*Zd2')*(Xj2-ones(n2,1).*beta02f(j)))/2;
                b2f = b2_p+((Xj2f-beta02f(j))*(1/(1+Zd2fs*V*Zd2fs'))*(Xj2f-beta02f(j))+M'*(Vinv-Vinv*inv(Vinv+Zd2fs'*Zd2fs)*Vinv)*M)/2;                
                BB = BB*det(V)^(-0.5)*det(Zd2fs'*Zd2fs+inv(V))^(-0.5)*(b2_p^ak_p2)/(b2f^akf2)*gamma(akf2)/gamma(ak_p2);
                
            end
            likf1 = (n1/n)*AA; likf2 = (n2/n)*BB;      
            Ppost = [likf1 likf2]; PpostM = [PpostM; Ppost./sum(Ppost)];
            ClassP = [ClassP find(Ppost==max(Ppost))];
        end       
        err = sum((ClassP-1-(Yf)')~=0);
        numErroriV = [numErroriV err ]; numVarV = [ numVarV sum(gammaf)];
    end
    numErrori(jj,:) = numErroriV; numVar(jj,:) = numVarV ; numSnp1(jj,:) = nSNPsel1; numSnp2(jj,:) = nSNPsel2;
    display(nSNPsel1)
    display(nSNPsel2)
    display(numErroriV)
    display(numVarV)
end

disp(' ')
disp('------- number of miss-classified subjects...')
disp(' ')
display(numErrori)

disp(' ')
disp('------- Miss-classification rate...')
disp(' ')
display(numErrori/length(Yf))

save filename.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

